from collections import OrderedDict

from django.contrib import messages
from django.contrib.auth.views import REDIRECT_FIELD_NAME, LoginView
from django.core.exceptions import PermissionDenied
from django.shortcuts import redirect
from django.utils.decorators import method_decorator
from django.views.decorators.cache import never_cache

from .forms import LoginForm


class CruditorMixin(object):
    """
    Base mixin for all Cruditor views. Provides common functionality for all views.

    It is a good idea to have a own "base" mixin to configure the common options
    like ``menu_title``, the urls and templates.

    Usually you might have ``required_permission`` configured per-view.
    """

    #: Title to use in templates / menu bar
    menu_title = 'CRUDitor'

    #: URL to use in the linked menu_title
    index_url = '#'

    #: URL to use when providing a logout link to the user.
    logout_url = '#'

    #: URL to the change password view, if available.
    change_password_url = None

    #: Template name which is included to render the menu.
    menu_template_name = 'cruditor/includes/menu.html'

    #: Template used to include extra head stuff.
    extrahead_template_name = 'cruditor/includes/extrahead.html'

    #: Page template for the login view.
    login_template_name = 'cruditor/login.html'

    #: Form class which is used in the login view.
    login_form_class = LoginForm

    #: Decide if only staff users should be able to use the Cruditor views.
    staff_required = True

    #: Which permission is required to access the view.
    required_permission = None

    #: If not provided, Cruditor tries to look up the verbose name from ``model.Meta``
    model_verbose_name = None

    @method_decorator(never_cache)
    def dispatch(self, request, *args, **kwargs):
        """
        Ensure the user is logged in (by calling `ensure_logged_in`` method).
        If the user is logged in, permissions are checked by calling
        ``ensure_required_permission``.
        """
        login_result = self.ensure_logged_in(request, *args, **kwargs)
        if login_result is not True:
            return login_result

        self.ensure_required_permission()

        return super().dispatch(request, *args, **kwargs)

    def get_cruditor_context(self, alternative_title=None, login_context=False):
        """
        Provides some context for all Cruditor templates to render menu, header,
        breadcrumb and title buttons.

        The method takes an optional argument ``alternative_title`` to override
        the default title from ``get_title`` method.
        """
        constants = {
            'menu_title': self.menu_title,
            'menu_template_name': self.menu_template_name,
            'extrahead_template_name': self.extrahead_template_name,
            'index_url': self.index_url,
            'logout_url': self.logout_url,
            'change_password_url': self.change_password_url,
        }

        if login_context:
            return {
                'title': 'Login',
                'constants': constants,
            }

        return {
            'title': alternative_title or self.get_title(),
            'breadcrumb': self.get_breadcrumb()
            + [{'title': alternative_title or self.get_breadcrumb_title(), 'url': None}],
            'titlebuttons': self.get_titlebuttons(),
            'constants': constants,
        }

    def get_title(self):
        """
        Returns the title of the page. Uses view's ``title`` property. If not set
        falls back to ``menu_title``.
        """
        return getattr(self, 'title', self.menu_title)

    def get_breadcrumb_title(self):
        """
        By default, the breadcrumb title is the same as the page title.
        Calls ``get_title`` if not overwritten.
        """
        return self.get_title()

    def get_breadcrumb(self):
        """
        This method is expected to return a list of breadcrumb elements as a list.

        Every breadcrumb element is a dict or object with at least
        a ``title`` property/key. If a ``url`` key/property is provided, the item
        is linked.
        """
        return []

    def get_titlebuttons(self):
        """
        This method is expected to return None or a list of buttons to display in
        the title row of the page.

        Every button element is a dict or object with at least a ``label`` and
        ``url`` property/key. In addition, one can provide an alternative
        ``button_class`` which is used as a css class - pefixed with "btn-".
        Default ``button_class`` is "light".
        """
        return None

    def get_model_verbose_name(self):
        """
        Returns the verbose name of the handled object/item.

        If ``model_verbose_name`` is set, the value is used. If not, Cruditor
        tries to get the verbose name from the model property (via Meta class).
        If no name is available at all, "Item" is returned.
        """
        if self.model_verbose_name:
            return self.model_verbose_name
        if getattr(self, 'model', None):
            return self.model._meta.verbose_name
        return 'Item'

    def ensure_logged_in(self, request, *args, **kwargs):
        """
        This method checks if the request user is logged in and has the right
        flags set (e.g. ``is_staff`` if ``staff_required`` is set in view).

        If user is logged in, ``True`` is returned.
        If not, ``handle_not_logged_in`` is called.
        """
        if not request.user.is_active or (self.staff_required and not request.user.is_staff):
            return self.handle_not_logged_in(request, *args, **kwargs)

        return True

    def handle_not_logged_in(self, request, *args, **kwargs):
        """
        This method is responsible to handle not logged-in users.
        By default, renders the Django login view using a Cruditor optimized
        template using the ``login_form_class`` as Form.
        """
        return LoginView.as_view(
            template_name=self.login_template_name,
            redirect_field_name=REDIRECT_FIELD_NAME,
            form_class=self.login_form_class,
            redirect_authenticated_user=False,
            extra_context={
                'app_path': request.get_full_path(),
                'next_field': REDIRECT_FIELD_NAME,
                'next_value': request.get_full_path(),
                'cruditor': self.get_cruditor_context(login_context=True),
            },
        )(request)

    def get_required_permission(self):
        """
        Returns the required Django permissions required to access the view.

        You might override the method to apply more complex rules on what
        permissions are required.
        """
        return self.required_permission

    def ensure_required_permission(self):
        """
        This method ensures that all required permissions (fetched by calling
        ``get_required_permission``).

        If permissions are not met, ``PermissionDenied`` is raised.
        """
        required_permission = self.get_required_permission()

        if not required_permission:
            return

        if not self.request.user.has_perm(required_permission):
            raise PermissionDenied

    def get_context_data(self, **kwargs):
        """
        Adds the ``cruditor`` context variable to the template context. Uses data
        from ``get_cruditor_context`` method.
        """
        context = super().get_context_data(**kwargs)
        context['cruditor'] = self.get_cruditor_context()
        return context


class FormViewMixin(object):
    """
    Mixin to add formset support to Django FormViews. To use formsets,
    you have to provide a set of formsets as a dict (or OrderedDict if you have
    more than one formset - just to have a defined ordering).
    """

    formset_classes = None

    def get_formset_classes(self):
        """
        This method returns the formset classes to render in the form view.
        By default, returns the ``formset_classes`` property.
        """
        return self.formset_classes or {}

    def get(self, request, *args, **kwargs):
        """
        Extended get-method to render to form and all formsets properly initialized.
        """
        self.object = self.get_object()

        formsets = OrderedDict(
            [
                (formset_name, formset_class(instance=self.object))
                for formset_name, formset_class in self.get_formset_classes().items()
            ]
        )

        return self.render_to_response(
            self.get_context_data(
                form=self.get_form(self.get_form_class()),
                formsets=formsets,
            )
        )

    def post(self, request, *args, **kwargs):
        """
        Extended version of the FormView.post method which validates the form and
        all configured formsets. If everything is valid, ``form_valid`` is called.
        If something is not valid, ``form_invalid`` is called.

        Both the form instance and all formset instances are provided to the called
        method. The form is passed as the first argument, the formsets are passed
        as keyword arguments using the formset key from ``formset_classes``.
        """
        self.object = self.get_object()

        form = self.get_form(self.get_form_class())
        formsets = OrderedDict(
            [
                (
                    formset_name,
                    formset_class(request.POST, files=request.FILES, instance=self.object),
                )
                for formset_name, formset_class in self.get_formset_classes().items()
            ]
        )

        if all([form.is_valid()] + [formset.is_valid() for formset in formsets.values()]):
            return self.form_valid(form, **formsets)
        else:
            return self.form_invalid(form, **formsets)

    def save_form(self, form, **formsets):
        """
        This method is called from ``form_valid`` to actual save the data from the
        form and all formsets. All saving is done by default.
        """
        self.object = form.save()

        self.formset_objects = {}
        for formset_name, formset in formsets.items():
            formset.instance = self.object
            self.formset_objects[formset_name] = formset.save()

    def get_success_message(self):
        """
        Returns the success message to display when the form is valid.
        """
        return self.success_message.format(
            model=self.get_model_verbose_name(), object=self.object
        )

    def form_valid(self, form, **formsets):
        """
        Saves the data and provides a nice success message, then redirects to the
        ``get_success_url`` url.
        """
        self.save_form(form, **formsets)

        messages.success(self.request, self.get_success_message())

        return redirect(self.get_success_url())

    def form_invalid(self, form, **formsets):
        """
        Re-render the page with the invalid form and/or formsets.
        """
        return self.render_to_response(
            self.get_context_data(
                form=form,
                formsets=formsets,
                formset_errors=True,
            )
        )



from django import forms
from django.contrib.auth.forms import AuthenticationForm, SetPasswordForm
from django.forms.formsets import DELETION_FIELD_NAME
from django.utils.functional import cached_property
from django.utils.translation import gettext
from tapeforms.contrib.bootstrap import BootstrapTapeformMixin


class CruditorTapeformMixin(BootstrapTapeformMixin):
    """
    Cruditor mixin for all forms (relies on django-tapeforms).
    """

    pass


class CruditorFormsetMixin(object):
    """
    Helper mixin to provide some additional configuration to the javascript part
    of Cruditor's formset support, mainly translations but also all other stuff
    which might be needed.
    """

    js_formset_options = None

    @cached_property
    def template_context(self):
        """
        This cached property can be used to access extra context in the formset template
        while keeping the performance up by not "generating" the context over and over
        again. The return value of this property is generated by calling the method
        `get_template_context` once per formset instance.
        """

        return self.get_template_context()

    def get_template_context(self):
        """
        This method builds a context which is used by the `template_context` property to
        return additional context when rendering the formset. Some defaults are already set
        and might be overwritten.
        """
        return {
            'form_headline_new_instance': gettext('New item'),
        }

    def add_fields(self, form, index):
        """
        Overwritten method to make sure the DELETE marker field is hidden in output.
        """
        super().add_fields(form, index)

        if DELETION_FIELD_NAME in form.fields:
            form.fields[DELETION_FIELD_NAME].widget = forms.HiddenInput()

    def get_js_formset_options(self):
        """
        This method builds the options dict for the javascript part. Some defaults
        are merged with `js_formset_options` property.
        """
        options = {
            'prefix': self.prefix,
            'add-button-label': gettext('Add another'),
            'add-title': gettext('New item'),
            'delete-button-label': gettext('Delete item'),
            'delete-confirm-text': gettext('Are you sure? Item will be deleted after saving.'),
        }
        options.update(self.js_formset_options or {})
        return options


class CruditorFormsetFormMixin(CruditorTapeformMixin):
    """
    Helper mixin for forms in a formset, used together with Cruditor-enabled
    formsets.
    """

    def visible_fields(self):
        """
        This method is overwritten to  make sure that the DELETE marker field
        is not considered when returning the list of visible fields.
        """
        return [
            field for field in super().visible_fields() if field.name != DELETION_FIELD_NAME
        ]

    def hidden_fields(self):
        """
        This method is overwritten to  make sure that the DELETE marker field
        is not considered when returning the list of hidden fields.
        Cruditor template renders the field manually.
        """
        return [field for field in super().hidden_fields() if field.name != DELETION_FIELD_NAME]


class LoginForm(CruditorTapeformMixin, AuthenticationForm):
    """
    Tapeform-enabled version of the Django AuthenticationForm.
    """

    pass


class ChangePasswordForm(CruditorTapeformMixin, SetPasswordForm):
    """
    Tapeform-enabled version of the Django SetPasswordForm.
    """

    pass



import functools
import operator

from django.db.models import Q
from django.utils.translation import gettext
from django_filters import CharFilter, ChoiceFilter


class AnyChoiceFilter(ChoiceFilter):
    """
    Extended ChoiceFilter which adds an "any" choice to the choices from the
    field / provided options by setting a empty_label on the generated form field.
    """

    def __init__(self, *args, **kwargs):
        empty_label = kwargs.pop('empty_label', gettext('Any choice'))
        super().__init__(*args, **kwargs)
        self.extra['empty_label'] = empty_label


class MultiCharFilter(CharFilter):
    """
    This filter performs an OR query on the defined fields from a
    single entered value.

    The following will work similar to the default UserAdmin search::

        class UserFilterSet(FilterSet):
            search = MultiCharFilter([
                'username', 'first_name', 'last_name', '^email'])

            class Meta:
                model = User
                fields = ['search']

    The filter supports filtering in different modes (icontains, istartswith,
    iexact, and search). icontains is the default mode, use ^, = and @ in the
    list of fields for the other modes.

    Based on some ideas from https://gist.github.com/nkryptic/4727865
    """

    default_lookup_type = 'icontains'
    lookup_types = [
        ('^', 'istartswith'),
        ('=', 'iexact'),
        ('@', 'search'),
    ]

    def __init__(self, fields, *args, **kwargs):
        self.fields = fields
        super().__init__(*args, **kwargs)

    def filter(self, qs, value):
        if not self.fields or not value:
            return qs

        lookups = [self._get_lookup(str(field)) for field in self.fields]
        queries = [Q(**{lookup: value}) for lookup in lookups]
        qs = qs.filter(functools.reduce(operator.or_, queries))
        return qs

    def _get_lookup(self, field_name):
        for key, lookup_type in self.lookup_types:
            if field_name.startswith(key):
                return '{0}__{1}'.format(field_name[len(key) :], lookup_type)
        return '{0}__{1}'.format(field_name, self.default_lookup_type)



from django.contrib import messages
from django.contrib.auth import update_session_auth_hash
from django.contrib.auth.views import LogoutView
from django.core.exceptions import ImproperlyConfigured
from django.db import models
from django.http import HttpResponseRedirect
from django.shortcuts import redirect
from django.utils.decorators import method_decorator
from django.utils.text import capfirst
from django.utils.translation import gettext
from django.utils.translation import gettext_lazy as _
from django.views.decorators.cache import never_cache
from django.views.generic import CreateView, DeleteView, FormView, TemplateView, UpdateView

from .forms import ChangePasswordForm
from .mixins import CruditorMixin, FormViewMixin


try:
    import django_tables2 as tables
except ImportError:
    # We don't want to break if django-tables2 is not available. Users would have
    # to overwrite the get_table method if they don't want to use django-tables2.
    tables = None


class Cruditor404View(CruditorMixin, TemplateView):
    """
    Customized not found page. Needed to add the required cruditor context
    for properly rendered templates.
    """

    #: Template used to render the 404 page.
    template_name = 'cruditor/404.html'

    @method_decorator(never_cache)
    def dispatch(self, request, *args, **kwargs):
        context = self.get_context_data(**kwargs)
        return self.render_to_response(context, status=404).render()


class Cruditor403View(CruditorMixin, TemplateView):
    """
    Customized permission denied page. Needed to add the required cruditor
    context for properly rendered templates.
    """

    #: Template used to render the 403 page.
    template_name = 'cruditor/403.html'

    @method_decorator(never_cache)
    def dispatch(self, request, *args, **kwargs):
        context = self.get_context_data(**kwargs)
        return self.render_to_response(context, status=403).render()


class CruditorListView(CruditorMixin, TemplateView):
    """
    Enhanced list view backed by django-tables2 and django-filters.

    You want to set at least the ``model`` property. The remaining default
    property values are just fine for a working output.

    By providing a alternative ``table_class`` and/or ``filter_class`` you can
    improve the usability of the view even further.
    """

    #: Model to work on in this view.
    model = None

    #: Queryset to use for looking up objects.
    queryset = None

    #: Optional django_filters.FilterSet class to provide filtering capabilities.
    filter_class = None

    #: Required django_tables2.Table class to change the rendered table.
    table_class = None

    #: Template to use when rendering the list view.
    template_name = 'cruditor/list.html'

    def get_context_data(self, **kwargs):
        """
        Prepares the context by adding the ``table`` context variable.
        If you have configured ``filter_class``, the ``filter_form`` context
        variable will be provided too.
        """
        context = super().get_context_data(**kwargs)
        filtered_qs = self.get_filtered_queryset()
        context['table'] = self.get_table(filtered_qs)
        context['filter_form'] = filtered_qs.form if hasattr(filtered_qs, 'form') else None
        return context

    def get_queryset(self):
        """
        Provide a queryset to fetch data with. If ``queryset`` is set on the class,
        the value will be used, if not but ``model`` is set, the default manager
        is used.

        If both queryset and model is not set, you have to override this method
        to provide data to display.
        """
        if self.queryset is not None:
            return self.queryset

        if getattr(self, 'model', None):
            return self.model._default_manager.all()

        return []

    def get_table_class(self):
        """
        Method to override the used table class. By default, returns ``table_class``
        property if set.

        If no ``table_class``is provided, an ``ImproperlyConfigured`` exception is raised.
        """
        if not self.table_class:
            raise ImproperlyConfigured('table_class not configured.')
        return self.table_class

    def get_table_kwargs(self):
        """
        Override to provide additional kwargs when initializing the table object.
        """
        return {}

    def get_filter_class(self):
        """
        Method to override the used filter class from django_filters. By default,
        returns ``filter_class`` property if set.
        """
        return self.filter_class

    def get_filter_kwargs(self):
        """
        Override to provide additional kwargs when initializing the filterset object.
        """
        return {}

    def get_filtered_queryset(self):
        """
        Filter the base queryset using the django-filters FilterSet if available.
        QuerySet is passed if no ``filter_class`` is defined.
        """
        filter_class = self.get_filter_class()
        if filter_class:
            return filter_class(
                self.request.GET, queryset=self.get_queryset(), **self.get_filter_kwargs()
            )

        return self.get_queryset()

    def get_table(self, filtered_qs):
        """
        Prepare the table object using the provided QuerySet/Iterable.
        """
        qs = getattr(filtered_qs, 'qs', filtered_qs)
        if hasattr(qs, 'distinct'):
            qs = qs.distinct()
        table = self.get_table_class()(qs, **self.get_table_kwargs())
        tables.RequestConfig(self.request).configure(table)
        return table


class CruditorAddView(CruditorMixin, FormViewMixin, CreateView):
    """
    Enhanced view to add new items using a form view.
    """

    #: Message used when a new item was added successfully.
    success_message = _('The {model} "{object}" was successfully added.')

    #: Template used to render the add form view.
    template_name = 'cruditor/form.html'

    def get_object(self):
        """
        As we are in a add view, no object will be available ever.
        """
        return None

    def get_title(self):
        """
        Generate a sane title when adding new items using the
        ``get_model_verbose_name``.
        """
        return gettext('Add {0}').format(self.get_model_verbose_name())


class CruditorChangeView(CruditorMixin, FormViewMixin, UpdateView):
    """
    Enhanced view to edit existing items using a form view.
    """

    #: Message used when a item was changed successfully.
    success_message = _('The {model} "{object}" was successfully changed.')

    #: Template used to render the change form view.
    template_name = 'cruditor/form.html'

    def get_title(self):
        """
        Generate a sane title when editing an item using the __str__
        representation of a object.
        """
        return gettext('Change: {0}').format(self.object)

    def get_delete_url(self):
        """
        Override to provide a link for the delete button in the change view.
        By default no delete button is visible.
        """
        return None

    def get_context_data(self, **kwargs):
        """
        Add the ``object_delete_url`` context variable using ``get_delete_url``.
        Feel free to extend the context further.
        """
        return super().get_context_data(object_delete_url=self.get_delete_url(), **kwargs)


class CruditorDeleteView(CruditorMixin, DeleteView):
    """
    Enhanced view to delete existing items after a confirmation.
    """

    #: Message used when a item was deleted.
    success_message = _('The {model} "{object}" was successfully deleted.')

    #: Template used to render the confirmation form view.
    template_name = 'cruditor/delete.html'

    def delete(self, request, *args, **kwargs):
        """
        Call ``perform_delete`` method and redirect to the success URL with a
        nice success message. If there are protected related objects, an error
        message is shown instead with the output of ``format_linked_objects``.
        """
        self.object = self.get_object()
        try:
            self.perform_delete()
        except models.ProtectedError as e:
            return self.render_to_response(
                self.get_context_data(
                    linked_objects=self.format_linked_objects(e.protected_objects)
                )
            )
        messages.success(
            self.request,
            self.success_message.format(
                model=self.get_model_verbose_name(), object=self.object
            ),
        )
        return HttpResponseRedirect(self.get_success_url())

    def get_title(self):
        """
        Generate a sane title when requesting a confirmation to delete an item
        using the __str__ representation of a object.
        """
        return gettext('Delete: {0}').format(self.object)

    def perform_delete(self):
        """
        Actual delete a object/model/item after confirmation.
        """
        self.object.delete()

    def format_linked_objects(self, objects):
        """
        Generate a list of strings describing the objects which have a protected
        relation to the item to delete.
        """
        return [
            gettext('{model}: {object}').format(
                model=capfirst(obj.__class__._meta.verbose_name), object=obj
            )
            for obj in objects
        ]


class CruditorChangePasswordView(CruditorMixin, FormView):
    """
    Enhanced view to perform password changes in the Cruditor context.
    """

    #: Template used when rendering the change password form.
    template_name = 'cruditor/form.html'

    #: Title for breadcrumb and page.
    title = _('Change password')

    #: Form used to change the password.
    form_class = ChangePasswordForm

    def get_form_kwargs(self):
        """
        The current user is passed to the provided ``form_class`` when initializing
        the change password form.
        """
        kwargs = super().get_form_kwargs()
        kwargs['user'] = self.request.user
        return kwargs

    def form_valid(self, form):
        """
        Save the new password (by calling ``form.save``) and rotate the session
        authorization hash.
        """
        form.save()
        update_session_auth_hash(self.request, form.user)
        messages.success(self.request, gettext('Password changed successfully.'))
        return redirect(self.request.path)

    def get_context_data(self, **kwargs):
        """
        Set ``form_save_button_label`` context variable to change the button
        label for the change password form.
        """
        return super().get_context_data(
            form_save_button_label=gettext('Set new password'), **kwargs
        )


class CruditorLogoutView(CruditorMixin, LogoutView):
    """
    View to log out the current user. After logging out, a info is displayed.
    """

    #: Template used to display the info that the user was logged out.
    template_name = 'cruditor/logout.html'

    def get_context_data(self, **kwargs):
        return super().get_context_data(
            cruditor=self.get_cruditor_context(alternative_title='Logout'), **kwargs
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['cruditor']

package_data = \
{'': ['*'],
 'cruditor': ['locale/de/LC_MESSAGES/*',
              'locale/fr/LC_MESSAGES/*',
              'static/cruditor/css/*',
              'static/cruditor/js/*',
              'templates/cruditor/*',
              'templates/cruditor/forms/*',
              'templates/cruditor/includes/*']}

install_requires = \
['Django>=2.2']

extras_require = \
{':python_version < "3.8"': ['importlib-metadata'],
 'docs': ['Sphinx>=3.5',
          'django-tapeforms>=1.1.0',
          'django-tables2>=2.4.1',
          'django-filter>=21.1']}

setup_kwargs = {
    'name': 'django-cruditor',
    'version': '2.0.0',
    'description': 'A set of class based views and mixins to generate CRUD interfaces.',
    'long_description': "django-cruditor\n===============\n\n.. image:: https://img.shields.io/pypi/v/django-cruditor.svg\n   :target: https://pypi.org/project/django-cruditor/\n   :alt: Latest Version\n\n.. image:: https://github.com/stephrdev/django-cruditor/workflows/Test/badge.svg?branch=master\n   :target: https://github.com/stephrdev/django-cruditor/actions?workflow=Test\n   :alt: CI Status\n\n.. image:: https://codecov.io/gh/stephrdev/django-cruditor/branch/master/graph/badge.svg\n   :target: https://codecov.io/gh/stephrdev/django-cruditor\n   :alt: Coverage Status\n\n.. image:: https://readthedocs.org/projects/django-cruditor/badge/?version=latest\n   :target: https://django-cruditor.readthedocs.io/en/stable/?badge=latest\n   :alt: Documentation Status\n\n\nUsage\n-----\n\nPlease refer to the `Documentation <https://django-cruditor.readthedocs.io/>`_ to\nlearn how to use ``django-cruditor``. Cruditor is a set of generic class based views\nwith UIKit styled templates. Together with django-tables2, django-filter and\ndjango-tapeforms this package provides you some easy to use Django views to build\nyour customized CRUD interface.\n\n\nRequirements\n------------\n\ndjango-cruditor supports Python 3 only and requires at least Django 2.2.\nOptional dependencies are django-tapeforms, django-tables2 and django-filter.\n\n\nPrepare for development\n-----------------------\n\nA Python 3 interpreter is required in addition to poetry.\n\n.. code-block:: shell\n\n    $ poetry install\n    # If you want to build docs, execute this in addition\n    $ poetry install -E docs\n\n\nNow you're ready to start the example project to experiment with cruditor.\n\n.. code-block:: shell\n\n    $ poetry run python examples/manage.py runserver\n",
    'author': 'Stephan Jaekel',
    'author_email': 'steph@rdev.info',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/stephrdev/django-cruditor',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.6.2,<4',
}


setup(**setup_kwargs)



try:
    import importlib.metadata as importlib_metadata
except ModuleNotFoundError:
    # This is required for Python versions < 3.8
    import importlib_metadata

try:
    __version__ = importlib_metadata.version('django-cruditor')
except Exception:
    __version__ = 'HEAD'
